# Admissions Portal

This page gives an overview of the Admissions Portal, describing its current status, where and
how it's developed and deployed, and who is responsible for maintaining it.

!!! note
    This service only applies to undergraduate applicants. The
    [Applicant Portal for postgraduates](https://apply.postgraduate.study.cam.ac.uk/) is an
    unrelated web application.

## Service Description

The Admissions Portal service provides a web application for undergraduate applicants to supply
supplementary information as part of their university application, and to track the progress of
their university application.

## Service Status

The Admissions Portal is currently alpha.

## Contact

Technical queries and support should be directed to
[the DevOps Hopper team](mailto:uis-devops-hopper@lists.cam.ac.uk
) and will be picked up by a member of
the team working on the service.

Issues discovered in the service or new feature requests should be opened as GitLab issues
[here](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/operational-support-and-helpdesk).

## Environments

The Admissions Portal is currently deployed to the following environments:

| Name        | Public Application URL | Data Service API URL | Payment Callback API URL |
| ----------- | ---------------------- | -------------------- | ------------------------ |
| Production  | [https://apply.undergraduate.study.cam.ac.uk](https://apply.undergraduate.study.cam.ac.uk) | [https://api.apps.cam.ac.uk/undergraduate-admissions/v1alpha1](https://api.apps.cam.ac.uk/undergraduate-admissions/v1alpha1) | [https://wpmapp.prod.ap.gcp.uis.cam.ac.uk/](https://wpmapp.prod.ap.gcp.uis.cam.ac.uk/) |
| User testing | [https://webapp.uat.ap.gcp.uis.cam.ac.uk/](https://webapp.uat.ap.gcp.uis.cam.ac.uk/) | n/a | [https://wpmapp.uat.ap.gcp.uis.cam.ac.uk/](https://wpmapp.uat.ap.gcp.uis.cam.ac.uk/) |
| Staging     | [https://webapp.test.ap.gcp.uis.cam.ac.uk/](https://webapp.test.ap.gcp.uis.cam.ac.uk/) | [https://api.apps.cam.ac.uk/undergraduate-admissions-test/v1alpha1](https://api.apps.cam.ac.uk/undergraduate-admissions-test/v1alpha1) | [https://wpmapp.test.ap.gcp.uis.cam.ac.uk/](https://wpmapp.test.ap.gcp.uis.cam.ac.uk/) |
| Development | [https://webapp.devel.ap.gcp.uis.cam.ac.uk/](https://webapp.devel.ap.gcp.uis.cam.ac.uk/) | [https://api.apps.cam.ac.uk/undergraduate-admissions-dev/v1alpha1](https://api.apps.cam.ac.uk/undergraduate-admissions-dev/v1alpha1) | [https://wpmapp.devel.ap.gcp.uis.cam.ac.uk/](https://wpmapp.devel.ap.gcp.uis.cam.ac.uk/) |

The GCP console pages for managing the infrastructure of each component of the deployment are:

| Name         | Public Application Static Assets | Public Application Backend | Internal Portal Backend and Payment Callback API | Frontend Cloud Functions | Data service backend |
| ------------ | -------------------------------- | -------------------------- | ----------------------- | ------------------------ |------------------- |
| Production   | [GCP CDN](https://console.cloud.google.com/net-services/cdn/list?project=ap-prod-43267ef1) | [Firebase](https://console.firebase.google.com/project/ap-prod-43267ef1) | [GCP Cloud Run](https://console.cloud.google.com/run?project=ap-prod-43267ef1) | [Cloud Functions](https://console.cloud.google.com/functions/list?project=ap-prod-43267ef1) | n/a |
| User testing | [GCP CDN](https://console.cloud.google.com/net-services/cdn/list?project=ap-uat-8c323712) | [Firebase](https://console.firebase.google.com/project/ap-uat-8c323712) | [GCP Cloud Run](https://console.cloud.google.com/run?project=ap-uat-8c323712) | [Cloud Functions](https://console.cloud.google.com/functions/list?project=ap-uat-8c323712) | n/a |
| Staging      | [GCP CDN](https://console.cloud.google.com/net-services/cdn/list?project=ap-test-3d41aff5) | [Firebase](https://console.firebase.google.com/project/ap-test-3d41aff5) | [GCP Cloud Run](https://console.cloud.google.com/run?project=ap-test-3d41aff5) | [Cloud Functions](https://console.cloud.google.com/functions/list?project=ap-test-3d41aff5) | [GCP Cloud Run](https://console.cloud.google.com/run?project=albatross-test-1a115fe1) |
| Development  | [GCP CDN](https://console.cloud.google.com/net-services/cdn/list?project=ap-devel-dff67399) | [Firebase](https://console.firebase.google.com/project/ap-devel-dff67399) | [GCP Cloud Run](https://console.cloud.google.com/run?project=ap-devel-dff67399) | [Cloud Functions](https://console.cloud.google.com/functions/list?project=ap-devel-dff67399) | [GCP Cloud Run](https://console.cloud.google.com/run?project=albatross-devel-7a2c4f75) |

All environments share access to a set of secrets stored in the [meta-project Secret Manager](https://console.cloud.google.com/security/secret-manager?project=ap-meta-296bac5a).

## Source code

The source code for the Admissions Portal is spread over the following repositories:

| Repository  | Description
| ----------- | ------------------ |
| [Portal Frontend](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/admissions-portal/ap-frontend) | The source code for the application frontend |
| [Internal Portal Backend](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/admissions-portal/ap-backend) | The source code for the internal backend service and WPM callback API |
| [Portal Infrastructure Deployment](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/admissions-portal/ap-deployment) | The Terraform infrastructure code for deploying the portal to GCP |
|[ Data service](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/albatross/albatross-backend) | The source code for the data service which provides an API on the [API Gateway](./api-gateway.md) |
| [Data service Deployment](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/albatross/albatross-deployment) | The Terraform infrastructure code for deploying the data service to GCP |

## Technologies used

The following gives an overview of the technologies the Admissions Portal is built on.

| Category | Language | Framework(s) |
| -------- | -------- | --------- |
| Frontend | JavaScript | React 17, Gatsby, Firestore |
| Cloud Functions | JavaScript | Nodejs 16 |
| Internal Backend API | Python 3.8 | FastAPI |
| WPM Callback API | Python 3.8 | FastAPI |
| Data Service API | Python 3.9 | FastAPI, PostgreSQL 14 |

## Operational documentation

The following gives an overview of how the Admissions Portal is deployed and maintained.

### How and where the Admissions Portal is deployed

The Admissions Portal infrastucture is deployed using Terraform, with releases of the frontend and
service-to-service backend API deployed by the GitLab CD pipelines associated with the
[infrastructure deployment repository](https://gitlab.developers.cam.ac.uk/uis/devops/digital-admissions/admissions-portal/ap-deployment).

### Deploying a new release

The `README.md` files in each of the source code repositories explain how to deploy the Admissions
Portal.

### Monitoring

[GCP Cloud Monitoring](https://console.cloud.google.com/monitoring?project=ap-meta-296bac5a)
is used for tracking the health of applications in the environments and sending alert emails when
problems are detected.

### Debugging

TBD

### Specifications

- [Admissions Portal: High-Level Specification](https://docs.google.com/document/d/1UJFCdnS06PpmILPfj75T0TTqnLQ7ZNEm-iIZxRVGXKU)
- [Admissions Portal: Backend Specification](https://docs.google.com/document/d/10aHuF78vrxITQkCLsUUUOmiodZOu7mvCI2_dBYbYgpk)
- [Admissions Portal: Frontend Specification](https://docs.google.com/document/d/1TKutBiVlW076sjX-EkM_nOTdOlZvSgssHDb4Y6czKPU)
- [Digital Admissions Project Index](https://docs.google.com/document/d/19j0wYtijlYHv3AYGKnSFSNASCSJT4saFEWFhQc3chgs)

## Service Management and tech lead

The **service owner** for the Admissions Portal is TBD.

The **service manager** for the Admissions Portal is TBD.

The **tech lead** for the Admissions Portal is
[Brent Stewart](https://www.lookup.cam.ac.uk/person/crsid/bs461).

The following engineers have operational experience with the Admissions Portal and are able to
respond to support requests or incidents:

* [Paul Horrocks](https://www.lookup.cam.ac.uk/person/crsid/pdh45)
* [Inês de Matos](https://www.lookup.cam.ac.uk/person/crsid/id353)
* [Alberto Morón Hernández](https://www.lookup.cam.ac.uk/person/crsid/amh254)